/**
  ******************************************************************************
  * @file    lcd_ks0108.c
  * @author  D. Erickson
  * @version V1.0
  * @date    02/16/2011
  * @brief   Graphics Library for KS0108 LCD on STM32
  *
  * Contains low level read and write, pixel, line and circle (ellipse) draw
  * routines, clear screen, character draw.
  *
  * Caveats:
  * The KS0108 is a cumbersome device for doing graphics on the display:
  * It takes hundreds of CPU cycles just to set one pixel, requiring a
  * read / modify / write of the display memory. Reads are particularly slow.
  * The good news is that there aren't many pixels to write.
  * The vertical data bytes are different than most LCDs. This requires that
  * font tables and bit maps be transposed and that characters be placed on
  * 8 pixel vertical boundaries.
  *
  * Since it has only 128 x 8 = 1K bytes of display memory it would be faster
  * and simpler to render the display in CPU memory and then transfer the
  * 1K bytes to the display.  However for larger displays it may be better
  * to render on the display. This is written so if / when you convert to a
  * larger display, the high level code can stay relatively unchanged.
  ******************************************************************************
  * @copy
  *
  * COPYRIGHT 2011 Erickson Engineering</center></h2>
  *
  * History:
  * 2/15/11	dje 	pixel, circle,  line, 5x7 font all working
  * 2/19/11 dje 	Removed delays from E pulse, replaced with __NOP()'s
  * Removed R/M/Ws from GPIO ops, replace with ->BSRR = ...
  * Next:
  * Add rect, larger fonts
  */

#include "lcd_ks0108.h"
#include "font5x7.h"
#include "stm32f10x_gpio.h"

void LCD_GPIOConfiguration(void)
{
	  GPIO_InitTypeDef GPIO_InitStructure;

	/* Set PortC to outputs for LCD control pins */
	  GPIO_InitStructure.GPIO_Pin = LCD_E_PIN | LCD_RS_PIN | LCD_RW_PIN | LCD_CS1_PIN | LCD_CS2_PIN ;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	  GPIO_Init(LCD_CTL_PORT, &GPIO_InitStructure);

	  /* Set PortC[7:0] to outputs for 8 bit data bus. Direction will be controlled as needed */
	  GPIO_InitStructure.GPIO_Pin = LCD_DB_PINS;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	  GPIO_Init(LCD_DB_PORT, &GPIO_InitStructure);
}


void lcd_Init(void)
{
	LCD_GPIOConfiguration();

	lcd_cmd(LCD_DISP_ON);
	lcd_cmd(LCD_START_0);
}


/** @brief Send command data to both LCD chips (CS1 and CS2 low)
 * It is faster and can't hurt.
 * Bus direction pins are always OUT except during a read. */
void lcd_cmd(unsigned char data)
{
//	LCD_DB_PORT->CRL = 0x33333333; 					// Data bus direction = output */
	LCD_DB_PORT->BSRR = 0x00FF0000 | data; 			// Output data */
	// CS1 and CS2 are low
	LCD_CTL_PORT->BRR = LCD_E_PIN | LCD_RS_PIN | LCD_RW_PIN |LCD_CS1_PIN | LCD_CS2_PIN; //Clear E, RW, RS=0
	pulse_E();
	Delay(7);
}


/** @brief Send data byte to one LCD chip.
 *   Bus direction always OUT except during a read.
 *   Send data using clever BSRR
 *   Send RS=1, CSn=0, RW=0
 *   Pulse E=1 E delays are in pulse_E()
 *   Delay() is to avoid wait for busy flag
 */
void lcd_wdata(unsigned char data, unsigned char chip)
{
//	LCD_DB_PORT->CRL = LCD_DB_OUT; 					/* Data bus direction = output */
	LCD_DB_PORT->BSRR = 0x00FF0000 | data; 			/* Output data */
	if (chip == 0)
	{
		LCD_CTL_PORT->BSRR = ((LCD_E_PIN | LCD_CS1_PIN | LCD_RW_PIN) << 16) |
		(LCD_CS2_PIN | LCD_RS_PIN);  				/*Clear E, CS2, RW = 0; Set CS1, RS */
	}
	else
	{
		LCD_CTL_PORT->BSRR = ((LCD_E_PIN | LCD_CS2_PIN | LCD_RW_PIN) << 16) |
		(LCD_CS1_PIN | LCD_RS_PIN) ; 				/*Clear E, CS1, RW=0;  Set CS2 , RS */
	}
	pulse_E();
	Delay(7);										/* Wait ~4 us to avoid BUSY read */
}


/** @brief Pulse E signal high then low.
 * Short delays to meet E timing setup and pulse width times. */
void pulse_E(void)
{
	asm("nop"); asm("nop"); asm("nop"); 						/* 140nS Setup time for control pins */
																/* But function call is enough */
	LCD_CTL_PORT->BSRR = LCD_E_PIN; 							/* pulse E HI for .5us */
	asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop");
	asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop");
	LCD_CTL_PORT->BRR = LCD_E_PIN; 								/* Clear E */
}


/** @brief  Read a data byte from the LCD memory
 * Requires a dummy read before the real read.
 * Sets the data bus direction to OUT when done. */
unsigned char lcd_rdata(unsigned char chip)
{
	unsigned char data;

	LCD_DB_PORT->CRL = LCD_DB_IN; 					/* Data bus direction = input */
	if (chip == 0)
	{
		LCD_CTL_PORT->BSRR = ((LCD_E_PIN | LCD_CS1_PIN) << 16) |
		(LCD_CS2_PIN | LCD_RS_PIN | LCD_RW_PIN); 	/*Clear E, CS2, Set CS1, RS, RW */
	}
	else
	{
		LCD_CTL_PORT->BSRR = ((LCD_E_PIN | LCD_CS2_PIN) << 16 ) |
		(LCD_CS1_PIN | LCD_RS_PIN | LCD_RW_PIN); 	/*Clear E, CS1, Set CS2 , RS, RW */
	}
	pulse_E();										/* The first E pulse is a dummy read */
	Delay(10);
	LCD_CTL_PORT->BSRR = LCD_E_PIN;  				/* pulse E HI for about 1uS */
	Delay(1);
	data = (unsigned char)LCD_DB_PORT->IDR;			/* Get the data */
	LCD_CTL_PORT->BRR = LCD_E_PIN; 					/* Clear E */
	Delay(10);

	LCD_DB_PORT->CRL = LCD_DB_OUT; 					/* Data bus dir. = out */
	return data;
}


/** @brief  Send data to chip at address set by x,y */
/* Send XADDR, YADDR, then Data  */
void lcd_w(unsigned char x, unsigned char y, unsigned char data)
{
	unsigned char chip = 0;
	lcd_cmd(LCD_XADDR | (x & 0x3F));
	lcd_cmd(LCD_YADDR | ((y>>3)& 0x07));
	if (x > 63)chip = 1;
	lcd_wdata(data, chip);
}


/** @brief Get read data from chip at address set by x,y */
/* Send XADDR, YADDR, then get data  */
unsigned char lcd_r(unsigned char x, unsigned char y)
{
	unsigned char chip = 0;
	lcd_cmd(LCD_XADDR | (x & 0x3F));
	lcd_cmd(LCD_YADDR | ((y>>3)& 0x07));
	if (x > 63) chip = 1;
	return (uint8_t)lcd_rdata(chip);
}


/** @brief Write a pixel at x, y. This is a s-l-o-w operation, lcd_w() is much faster
 * 	 Set the address registers
 *   Read the old data
 *   Mask the pixel
 *   Write it back
 */
void lcd_putpix( uint8_t x, uint8_t y, uint8_t data)
{
	uint8_t temp;
	uint8_t chip = 0;
	uint8_t mask;
	lcd_cmd(LCD_XADDR | (x & 0x3F));			/* Set X */
	lcd_cmd(LCD_YADDR | ((y>>3) & 0x07));       /* Set Y */
	mask = 1 << (y & 0x07); 					/* Create pixel mask */
	if (x > 63) chip = 1;						/* Determine which chip */
	temp = lcd_rdata(chip);
	lcd_cmd(LCD_XADDR | (x & 0x3F));			/* Set X again. Last read incremented it. */
	lcd_wdata(mask | temp, chip);
}


/** @brief  Clear the screen */
void lcd_clear(uint8_t value)
{
	int32_t y;
	for (y = 0; y < LCD_YSIZ; y+= 8)
	{
		lcd_clear_line(y, value);
	}
}

/** @brief  Clear a line  */
void lcd_clear_line(int32_t line, uint8_t value)
{
	int32_t x;

	for (x = 0; x < LCD_XSIZ; x++)
	{
		lcd_w(x, line, value);
	}
}


/** @brief Draw one character. You probably want to use lcd_string() instead
 * Characters are on 8 bit vertical boundaries. */
void lcd_char(uint8_t x, uint8_t y, char ch)
{
	char col;
	uint8_t coldata;
	uint8_t chip = 0;

	lcd_cmd(LCD_YADDR | ((y>>3)& 0x07));			/* Set Y just once */
	lcd_cmd(LCD_XADDR | (x & 0x3F));			    /* Set X address */
	for(col=0; col<FONTX; col++)
	{
		coldata = Font5x7[(FONTX * ch) + col]; 		/*Get font data */
		lcd_cmd(LCD_XADDR | (x & 0x3F));			/* X address needed only for 63-64 transition */
		if (x < 64)							 		/* see if right or left chip */
		{
			lcd_wdata(coldata, 0);
		}
		else if (x<128)
		{
			lcd_wdata(coldata, 1);
		}
		else										/* Do nothing if X>= 128 */
		{}
		x++;
	}
	if (x < 64)							 			/* see if right or left chip */
	{
		lcd_wdata(0, chip);
	}
	else
	{
		lcd_wdata(0, 1);
	}

}

/** Draw an ascii string of characters
 * Characters are on 8 bit vertical boundaries. */
void lcd_string(uint8_t x, uint8_t y, char *str)
{
	while (*str != 0)
	{
		if(*str >= 0x20)				/* Only display ASCII 0x20 and up */
		{
			lcd_char(x, y, *str++);
			x += FONT_SPACE;
		}
	}
	/* Clear to the end of the line.  This is a hack, usable only when
	 * you always write whole lines or when the display is cleared to the right.
	 * Or when the display is written from left to right.
	 * Faster and simpler than clearing each line before you write.
	 * No display flicker. */
	for ( ; x < LCD_XSIZ; x++)
	{
		lcd_w(x, y, 0);
	}
}

/*************************************************************************
 * lcd_line()
 * Inputs:
 *     horizontal and vertical start and end values
 * RETURNS:
 *     None
 * DESCRIPTION:
 *    Draw a line using Bresneham's algorithm
 * CAVEATS:
 *    Color is always '1' or black.
 *    No clipping of values
 ************************************************************************/
void lcd_line(int32_t startx,int32_t starty,int32_t endx,int32_t endy)
{
  int32_t x, y, i;
  int32_t e, xinc, yinc;
  int32_t dx, dy, dx2, dy2, dif;

   xinc = 1;
   yinc = 1;
   if (startx > endx)
   {
	   dx = startx;		/* swap x1,y1  with x2,y2 */
	   startx = endx;
	   endx = dx;
	   dx = starty;
	   starty = endy;
	   endy = dx;
   }
   dx = endx - startx;
   dy = endy - starty;
   x = startx;
   y = starty;

   if (dx < 0)
   {
	   xinc = -1;
	   dx = -dx;
   }
   if (dy < 0)
   {
	   yinc = -1;
	   dy = -dy;
   }

   dx2 = dx << 1;
   dy2 = dy << 1;

   /****************************************/

   if (dx > dy)
   {
	   dif = dy2 - dx2;
	   e = dy2 - dx;
	   for (i = 0; i <= dx; i++)
	   {
		   lcd_putpix(x, y, 1);
	      if (e > 0)
	      {
		      y += yinc;
		      e += dif;
	      }
	      else
		      e += dy2;
	      x += xinc;
	   }
   }

   else
   {
	   dif = dx2 - dy2;
	   e = dx2 - dy;
	   for (i = 0; i <= dy; i++)
	   {
		  lcd_putpix(x, y, 1);
	      if (e > 0)
	      {
		      x += xinc;
		      e += dif;
	      }
	      else
		      e += dx2;
	      y += yinc;
	   }
   }
}

/************************************************************************
 *  lcdCircle()
 *  RETURNS:
 *     	None
 *  DESCRIPTION:
 *     	Draws an ellipse with specified major and minor axes
 *  CAVEATS:
 * 		No checking for valid data is performed
 *    	Uses a lot of goto's
 ************************************************************************/
void lcdCircle( int32_t xc, int32_t yc, int32_t xrad, int32_t yrad)
/* horizontal center, vertical center, horizontal radius, vertical radius */
{
   int xi;			/* current horizontal coord. for plotting 		*/
   int yi;			/* "   vertical     "     "     "     			*/
   int deli;		/* intermediate calc. for point generation 		*/
   int rho;			/* "     "     "    "       "      				*/
   int rhop;		/* "     "     "    "       "      				*/
   int lim;			/* final y coordinate 							*/
   int ns;			/* scaled coordinate 							*/
   int n;			/* other scaled coordinate 						*/
   int inc;		    /* fixed point fraction = min(x/y, y/x) 		*/


   xi = 0;
   deli = (1 - xrad) << 1;
   lim = 0;
   if (xrad > yrad)
   {
	   yi = xrad;
	   inc = (yrad << 8) / xrad;
	   ns = yrad << 8;
	   while (yi >= lim)
	   {
	      n = ns >> 8;
	      lcd_putpix( xi + xc, yc + n, 1);
	      lcd_putpix( xi + xc, yc - n, 1);
	      lcd_putpix( xc - xi, yc + n, 1);
	      lcd_putpix( xc - xi, yc - n, 1);
	      if (deli > 0)
		      goto l3;
	      if (deli == 0)
		      goto l20;
	      rho = ((deli + yi) << 1) - 1;
	      if (rho <= 0)
		      goto l10;
	      else
		      goto l20;
l3:   rhop = ((deli - xi) << 1) - 1;
	   if (rhop <= 0)
		goto l20;
	   else
		goto l30;
l10:  ++xi;
	   deli += (xi << 1) + 1;
	   continue;
l20:  ++xi;
	   --yi;
	   ns -= inc;
	   deli += ((xi - yi + 1) << 1);
	   continue;
l30:  --yi;
	   ns -= inc;
	   deli += 1 - (yi << 1);
	   }
   }
   else
   {
	   yi = yrad;
	   inc = (xrad << 8) / yrad;
	   ns = 0;
	   while (yi >= lim)
      {
	      n = ns >> 8;
	      lcd_putpix( n + xc, yc + yi, 1);
	      lcd_putpix( n + xc, yc - yi, 1);
	      lcd_putpix( xc - n, yc + yi, 1);
	      lcd_putpix( xc - n, yc - yi, 1);
	      if (deli > 0)
		      goto l3a;
	      if (deli == 0)
		      goto l20a;
	      rho = ((deli + yi) << 1) - 1;
	      if (rho <= 0)
		      goto l10a;
	      else
		      goto l20a;
    l3a: rhop = ((deli - xi) << 1) - 1;
	      if (rhop <= 0)
		      goto l20a;
	      else
		      goto l30a;
 l10a:   ++xi;
	      ns += inc;
	      deli += (xi << 1) + 1;
	      continue;
    l20a:  ++xi;
	      ns += inc;
	      --yi;
	      deli += ((xi - yi + 1) << 1);
	      continue;
    l30a:  --yi;
	      deli += 1 - (yi << 1);
	      }
     }
}

/* Draw a 6 pixel high bar in the 8 pixel row.
 * In case the length is 0, just draw a vertical tick.
 * This function is specific to the KS0108's vertical pixel format.
 */
void lcd_bar(uint16_t xs, uint16_t ys, int16_t len)
{
	uint16_t x;

	if (len > 0)
	{
		for(x=xs; x < xs +len; x++)
		{
			lcd_w(x, ys, 0x7e);
		}
		lcd_w(xs, ys, 0xff);
	}
	if (len == 0)
		{
			lcd_w(xs, ys, 0xff);
		}
	if (len < 0)
	{
		for(x = xs + len; x < xs; x++)
		{
			lcd_w(x, ys, 0x7e);
		}
		lcd_w(xs, ys, 0xff);
	}
}

/* Draw a box of 4 lines */
void lcd_rect(uint16_t xs, uint16_t ys, uint16_t xsiz, uint16_t ysiz)
{
	lcd_line(xs, ys, xs + xsiz, ys       );				/* Top */
	lcd_line(xs, ys, xs              , ys + ysiz);		/* Left */
	lcd_line(xs, ys + ysiz, xs + xsiz, ys + ysiz);		/* Bot */
	lcd_line(xs + xsiz, ys, xs + xsiz, ys + ysiz);		/* Right */
}

/* Test: Display Lines and Circle. Assumes clear screen first. */
void lcdTest(void)
{
	int i;

	for(i=0; i<6; i++)
	{
	  lcd_line(12, 13, 100-2*i, 7*i);
	}
	lcdCircle( 40, 20, 20, 17);
}

/* Scrap-heap of LCD test code */

/* Test of LCD Reads */
//	  lcd_w(1,0, 0x42);
//	  lcd_w(5,0, 0x69);
//	  for (i=0; i<8; i++)
//	  {
//		  data[i] = lcd_r(i,0);
//	  }
//	  sprintf(buf1, "data= %x %x %x %x %x %x %x %x \n\r", data[0],data[1],data[2],data[3],data[4],data[5],data[6],data[7]);
//	  putString(buf1);

/* Test complete font table 16x8 chars */
//	  for(row=0; row<8; row++)
//	  {
//		  for(ch=0; ch<16; ch++)			// Test lcd_char
//		  {
//			  lcd_char(0 + (6 * ch), row*8, row*16 + ' ' + ch);
//		  }
//	  }

