/**
  ******************************************************************************
  * @file    Multi-Zone Preamp  main.c
  * @author  D. Erickson
  * @version V1
  * @date    02/18/2011
  * @brief   Main program: init and main loop are in main()
  * 		 Global variables and structs live here
  * 		 Use main.h where global variables are needed
  *
  * 		 The main control loop uses the 1mS time tick to control timing.
  * @history 03/19/11 dje  XPT board is working. First pass at the main data structure.
  * 		 04/11/11 dje  All controls: UART, Keypad, Remotes, IR, are working
  * 		 04/14/11 dje  Encoder knob working
  * 		 07/15/11 dje  Channels renamed, remote keypads moved to CH4-6
  *			 07/24/11 dje  inKey struct implemented.
  *			 07/30/11 dje  Fixed gain channels
  *			 07/30/11 dje  Mute off when any key is hit
  *			 08/07/11 dje  IR filter working, mute all added
  *			 01/01/12 dje  The Evil Atollic now limits code size to 32K
  *			 12/18/13 dje  Compiling under CooCox. Required updating stm32f110x_conf.h Happy!!
  *
  * @remaining
  *
  *          CooCox can't optimize (-01to4)due to an assembler bug in CMSIS. No optimization for now.
  *
  ******************************************************************************
  * @copy
  *
  */ 

/* Includes ------------------------------------------------------------------*/
#include <stdio.h>
#include "stm32f10x.h"
#include "uart.h"
#include "lcd_ks0108.h"
#include "main.h"
/* Don't forget to enable the appropriate lines in stm32f10x_conf.h */
#include "stm32f10x_conf.h"
#include "i2c.h"


/* Private typedef -----------------------------------------------------------*/

/* Private define ------------------------------------------------------------*/
#define DAC_DHR12RD_Address      0x40007420

/* Init Structure definition */
DAC_InitTypeDef            DAC_InitStructure;
DMA_InitTypeDef            DMA_InitStructure;
TIM_TimeBaseInitTypeDef    TIM_TimeBaseStructure;

/* Main control structures: Global */
PreampTypeDef  preamp[8];		/* Current state of the preamp zones */
inKeyTypeDef inKey[REMKEY0 + REMKEY_NUM + 1 ];		/* assign 1 per key type.  */
irTypeDef ir;					/* Control structure for IR */

/* Private macro -------------------------------------------------------------*/

/* Private variables ---------------------------------------------------------*/
/* Sine wave tableFor DAC */
const uint16_t Sine12bit[32] = {
       2047, 2447, 2831, 3185, 3498, 3750, 3939, 4056,
       4095, 4056, 3939, 3750, 3495, 3185, 2831, 2447,
       2047, 1647, 1263,  909,  599,  344,  155,   38,
          0,   38,  155,  344,  599,  909, 1263, 1647};

uint32_t DualSine12bit[32];
uint8_t Idx = 0;				/* Sine table index */

/* Scary global variables. Be very careful! */

/* For time tick */
uint32_t ms_count;
uint8_t sec_flag;
int32_t irCapture;
//int32_t irTimer;

uint16_t keypad_time;			/* Incremented by  1ms interrupt */
uint16_t ir_code;
uint16_t encoder_count;			/* the encoder */
uint16_t old_encoder;			/* the encoder */

#ifdef __GNUC__
  /* With GCC/RAISONANCE, small printf (option LD Linker->Libraries->Small printf
     set to 'Yes') calls __io_putchar() */
  #define PUTCHAR_PROTOTYPE int __io_putchar(int ch)
#else
  #define PUTCHAR_PROTOTYPE int fputc(int ch, FILE *f)
#endif /* __GNUC__ */

/* Private variables ---------------------------------------------------------*/
static __IO uint32_t TimingDelay;		/* Used for ms timer */

/**
  * @brief   Main program.
  * @param  None
  * @retval None
  */

int main(void)
{
  uint32_t cnt = 0;		/* Loop counter */
  uint8_t key;
  int32_t i;
  char buf1[64];		/* utility buffer for sprintf() */
  uint8_t zone, in_chan;	/* Temp only */
  uint8_t ledState = 0;

  /* Configure the hardware */

  /* System Clocks Configuration */
  RCC_Configuration();   
  /* NVIC configuration */
  NVIC_Configuration();
  /* Configure the GPIO ports */
  GPIO_Configuration();
  /* Configure TIM3 for the IR remote */
  TIM_Configuration();

  /* LCD startup and startup display */
  lcd_Init();
  lcd_clear(0);
  lcd_string(0, ROW_6, "Multi-Zone Stereo");
  lcd_string(0, ROW_7, "D. Erickson 2/18/13");
  lcdTest();

  /* DAC start-up */
  /* This is a totally unnecessary test of the DAC and DMA's ability to continuously
   * output a sine wave in the background with no processor intervention,
   * while the main program is running. You can use the sine wave as an audio test signal.  */

  DAC_Configuration();
  /* Fill Sine32bit table, invert LSBs */
  for (Idx = 0; Idx < 32; Idx++)
  {
    DualSine12bit[Idx] = (Sine12bit[Idx] << 16) + (0xFFFF ^ Sine12bit[Idx]);
  }

  /* ADC Configuration. ADC is used to read the remote keypads. */
   adc_chan = ADC_MIN;  		/*initialize the channel counter */
   adc_index = 0;
   ADC_Configuration();
   for(i=0; i<REMKEY_NUM; i++)
   {
	   remkey[i] = 0;
   }

  /* USART Configuration */
  USART_Configuration();

  /* Serial port wake-up message */
  sprintf(buf1, "\n\r\nMulti-Zone Preamp is awake! 2/18/13\n\r");
  putString(buf1);

  /* I2C Initialization */
  preamp_i2c_Init();
  MCP23008_init(I2C_XPT_ADDR );
  MCP23008_init(I2C_PRESEL0 );
#ifdef CHAN_8_BOARD
  MCP23008_init(I2C_PRESEL1 );
#endif

  Delay(6000000);			/* ~3 sec. delay before clearing the screen */
  lcd_clear(0);

/* Set up SysTick */
  if (SysTick_Config(SystemCoreClock / 1000))
  {
      while (1);      /* Capture error */
  }
/* Set up the various timers based on SysTick */
  ms_count = 999;		/* Timer counts down to 0 in the interrupt routine*/
  sec_flag = 0;

  /* Set up main preamp structure */
  initStruct();
  /* Init the crosspoint */
  xpt_init();
  /* Init the preamps */
  preamp_init();

/*  Set the initial zones and defaults */
  inKey[FPKEY].zone 				= 0;
  inKey[FPKEY].zoneDefault 			= 0;
  inKey[UARTKEY].zone 				= 0;
  inKey[UARTKEY].zoneDefault 		= 0;
  inKey[REMKEY0 + 0].zone 			= 4;
  inKey[REMKEY0 + 0].zoneDefault 	= 4;
  inKey[REMKEY0 + 1].zone 			= 5;
  inKey[REMKEY0 + 1].zoneDefault 	= 5;
  inKey[REMKEY0 + 2].zone 			= 6;
  inKey[REMKEY0 + 2].zoneDefault 	= 6;
  for(i = 0; i < (REMKEY0 + REMKEY_NUM); i++)	/* Zone timers */
  	{
  		inKey[i].zoneTimer = 0;
  	}

  zone = 0;
  update(zone);

  encoder_count = 0x8000;			/* Init the encoder knob count to mid scale. A hack until I properly code rollover*/
  old_encoder = encoder_count;

  while (1)			 /* Forever loop */
  {

	  /* read the front panel keypad */
	  key = key_proc(FPKEY);
	  if (key)
	  {
		  sprintf(buf1,"KP=%c \n\r", key); 		/* Display key */
		  putString(buf1);
		  dispatch(FPKEY, key);					 	/* Look it up and act */
		  update(inKey[FPKEY].zone);				/* Update the LCD */
	  }

	  /* Check IR */
	  key = key_proc(IRKEY);
	  if (key)
	  {
		  sprintf(buf1,"IR=%c \n\r", key); 		/* Display key */
		  putString(buf1);
		  dispatch(FPKEY, key);					/* Look it up and act */
		  update(inKey[FPKEY].zone);			/* Update the LCD */
		  inKey[FPKEY].zoneTimer = ZONE_TIMEOUT; /* Restart the zone timeout */
	  }
	  /* Display raw key stuff for IR filter debug*/
//	  sprintf(buf1,"RF=%d RC=%d ST=%d KC=%d\n\r",inKey[IRKEY].rawFlag,inKey[IRKEY].rawCode,inKey[IRKEY].state,inKey[IRKEY].keyCode);
//	  putString(buf1);

	  /* Check remote keypads */
	  for (i = 0; i < REMKEY_NUM; i++)
	  {
		  in_chan = REMKEY0 + i;
		  key = key_proc(in_chan);
		  if (key)
		  {
			  sprintf(buf1,"RK=%c \n\r", key); 	/* Display key */
			  putString(buf1);
			  dispatch(in_chan, key);			/* Look it up and act */
			  update(inKey[in_chan].zone);		/* Update the LCD */
		  }
	  }

	  /* Check the UART */
	  key = getChar(USART1);
	  if (key)
	  {
		  sprintf(buf1,"UA=%c \n\r", key); 		/* Display key */
		  putString(buf1);
		  dispatch(UARTKEY, key);				/* Look it up and act */
		  update(inKey[UARTKEY].zone);			/* Update the LCD */
		  inKey[UARTKEY].zoneTimer = ZONE_TIMEOUT; /* Restart the zone timeout */
	  }

	  /* Check the encoder */
	  key = encoder_proc();
	  if (key)
	  {
		  sprintf(buf1,"EN=%c \n\r", key); 	/* Display key */
		  putString(buf1);
		  dispatch(FPKEY, key);					/* Look it up and act */
		  update(inKey[FPKEY].zone);			/* Update the LCD */
		  inKey[FPKEY].zoneTimer = ZONE_TIMEOUT; /* Restart the zone timeout */
	  }

	  cnt++;								/* Increment the loop counter. Currently unused */

//	  Delay_ms(10);							/* This slows down the main loop every time */

	  /* Tasks to do once per second go here. */
	  if (sec_flag == 1)
	  {
		  if (ledState == 0)				/* Toggle the Blue LED */
		  {
			  LEDPORT->BSRR = BLUELED; 		/* Turn on LED */
			  ledState = 1;
		  }
		  else
		  {
			  LEDPORT->BRR = BLUELED; 		/* Turn off LED */
			  ledState = 0;
		  }

		  for(i = 0; i < (REMKEY0 + REMKEY_NUM); i++)		/* Zone timeouts. If timer == 0, set to default. */
		  {
			  if (inKey[i].zoneTimer > 0)
			  {
				  inKey[i].zoneTimer --;					/* Decrement once per second */
				  if ((inKey[i].zoneTimer == 0))			/* When any timer gets to 0, reset display to zone 0*/
				  {
					  inKey[i].zone = inKey[i].zoneDefault ;/* Currently only zone 0 uses this */
					  update(inKey[FPKEY].zone);			/* Display FPKEY zone */
				  }
			  }
			  if (inKey[i].zoneTimer == 0)
			  {
				  inKey[i].zone = inKey[i].zoneDefault ;
			  }
		  }

		  /* Debug: zone timer */
//		  sprintf(buf1, "FPTim=%d R0Tim=%d\n\r", inKey[FPKEY].zoneTimer, inKey[REMKEY0].zoneTimer);
//		  putString(buf1);

//		  ir_Test(); 				/* Debug: Display IR pulse widths here */

		  /* Debug: Test for remkey */
//		  sprintf(buf1, "RK0: %x RK1: %x RK2: %x\r\n",
//			remkey[0],remkey[1],remkey[2]);
//		  putString(buf1);

		  sec_flag = 0;				/* Clear the second flag */
	  } /* end of once per second stuff */
   }	/* end of while(1) loop */
}		/* end of main() */



/* These are the hardware configuration routines. */

/**
  * @brief  Configures the different system clocks.
  * @param  None
  * @retval None
  */
void RCC_Configuration(void)
{   
  /* Enable peripheral clocks ------------------------------------------------*/
#if !defined STM32F10X_LD_VL && !defined STM32F10X_MD_VL
  /* DMA2 clock enable */
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA2, ENABLE);
#else
  /* DMA1 clock enable */
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);
#endif
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_DAC, ENABLE);	// DACs
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);  // Timer 2
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);  // Timer 3
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1 , ENABLE); // ADC
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE); // GPIOA
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);	// GPIOB
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE); // GPIOC

#if defined (STM32F10X_LD_VL) || defined (STM32F10X_MD_VL)
  /* ADCCLK = PCLK2/2 */
  RCC_ADCCLKConfig(RCC_PCLK2_Div2);
#else
  /* ADCCLK = PCLK2/4 */
  RCC_ADCCLKConfig(RCC_PCLK2_Div4);
#endif
}

/**
  * @brief  Configure the nested vectored interrupt controller.
  * @param  None
  * @retval None
  */
void NVIC_Configuration(void)
{
  NVIC_InitTypeDef NVIC_InitStructure;

  /* Enable the TIM3 global Interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = TIM3_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
}


/**
  * @brief  Configures the GPIO ports.
  * @param  None
  * @retval None
  * 		Note: the LCD GPIO configuration is in LCD_GPIOConfiguration()
  */
void GPIO_Configuration(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  /* Port A pins 4 and 5 are DAC outputs. In order to avoid parasitic consumption,
     the pin should be configured as AIN */
  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_4 | GPIO_Pin_5;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  /* Set PortA[3:1] to Analog In for remote keypads*/
  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 ;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  /* TIM3 channel 2 pin (PA.07) configuration for IR remote control*/
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  /* Set PortB[1:0]to Analog In Remote keypads */
  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_0 | GPIO_Pin_1 ;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
  GPIO_Init(GPIOB, &GPIO_InitStructure);

  /* Set PortB[5:0] to outputs if needed.  Careful: B6, B7 are I2C */
  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_5|GPIO_Pin_4|
		  	  	  	  	  	  	  GPIO_Pin_3|GPIO_Pin_2;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
//  GPIO_Init(GPIOB, &GPIO_InitStructure);

  /* Set PortB[15:12] to digital inputs: keypad rows
   * and PortB[11:10] to inputs for the encoder knob*/
  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_15|GPIO_Pin_14|GPIO_Pin_13|GPIO_Pin_12|
		   	   	   	   	   	   	  GPIO_Pin_11|GPIO_Pin_10;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_Init(GPIOB, &GPIO_InitStructure);

  /* Set PortC[9,8] to outputs for Discovery board LEDs */
  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_9|GPIO_Pin_8;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_Init(GPIOC, &GPIO_InitStructure);
}


/* The DAC is used only to generate a continuous sine wave using DMA.
 * Really just a demo of the DAC and DMA, but I kept it in since
 * it's cool and can be used for testing the audio.
 */
void DAC_Configuration(void)
{
/* TIM2 Configuration */
  /* Time base configuration */
  TIM_TimeBaseStructInit(&TIM_TimeBaseStructure);
  TIM_TimeBaseStructure.TIM_Period = 250;        // 25 is about 1uS update  // 0x19
  TIM_TimeBaseStructure.TIM_Prescaler = 0x0;   // was 0x0
  TIM_TimeBaseStructure.TIM_ClockDivision = 0x0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);

  /* TIM2 TRGO selection */
  TIM_SelectOutputTrigger(TIM2, TIM_TRGOSource_Update);

  /* DAC channel1 Configuration */
  DAC_InitStructure.DAC_Trigger = DAC_Trigger_T2_TRGO;
  DAC_InitStructure.DAC_WaveGeneration = DAC_WaveGeneration_None;
  DAC_InitStructure.DAC_OutputBuffer = DAC_OutputBuffer_Disable;
  DAC_Init(DAC_Channel_1, &DAC_InitStructure);

  /* DAC channel2 Configuration */
  DAC_Init(DAC_Channel_2, &DAC_InitStructure);

#if !defined STM32F10X_LD_VL && !defined STM32F10X_MD_VL
  /* DMA2 channel4 configuration */
  DMA_DeInit(DMA2_Channel4);
#else
  /* DMA1 channel4 configuration */
  DMA_DeInit(DMA1_Channel4);
#endif
  DMA_InitStructure.DMA_PeripheralBaseAddr = DAC_DHR12RD_Address;
  DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)&DualSine12bit;
  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
  DMA_InitStructure.DMA_BufferSize = 32;
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Word;
  DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Word;
  DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
  DMA_InitStructure.DMA_Priority = DMA_Priority_High;
  DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;

#if !defined STM32F10X_LD_VL && !defined STM32F10X_MD_VL
  DMA_Init(DMA2_Channel4, &DMA_InitStructure);
  /* Enable DMA2 Channel4 */
  DMA_Cmd(DMA2_Channel4, ENABLE);
#else
  DMA_Init(DMA1_Channel4, &DMA_InitStructure);
  /* Enable DMA1 Channel4 */
  DMA_Cmd(DMA1_Channel4, ENABLE);
#endif

  /* Enable DAC Channel 1: Once DAC channel 1 is enabled, PA.04 is
     automatically connected to the DAC converter. */
  DAC_Cmd(DAC_Channel_1, ENABLE);
  /* Enable DAC Channel 2: Once DAC channel 2 is enabled, PA.05 is
     automatically connected to the DAC converter. */
  DAC_Cmd(DAC_Channel_2, ENABLE);
  /* Enable DMA for DAC Channel 2 */
  DAC_DMACmd(DAC_Channel_2, ENABLE);
  /* TIM2 enable counter */
  TIM_Cmd(TIM2, ENABLE);

}

void ADC_Configuration(void)
{
  ADC_InitTypeDef ADC_InitStructure;

  /* ADC1 configuration ------------------------------------------------------*/

/*  ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
  ADC_InitStructure.ADC_ScanConvMode = ENABLE;
  ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;
  ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
  ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
  ADC_InitStructure.ADC_NbrOfChannel = 1;
  ADC_Init(ADC1, &ADC_InitStructure);
  ADC_RegularChannelConfig(ADC1, ADC_Channel_1, 1, ADC_SampleTime_55Cycles5);
*/
  ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
  ADC_InitStructure.ADC_ScanConvMode = ENABLE;
  ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
  ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
  ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
  ADC_InitStructure.ADC_NbrOfChannel = 1;
  ADC_Init(ADC1, &ADC_InitStructure);

  ADC_RegularChannelConfig(ADC1, adc_chan, 1, ADC_SampleTime_55Cycles5);

  /* DMA not currently used. */
  /* Enable ADC1 DMA*/
  ADC_DMACmd(ADC1, ENABLE);

  /* Enable ADC1 */
  ADC_Cmd(ADC1, ENABLE);

  /* Enable ADC1 reset calibration register */
  ADC_ResetCalibration(ADC1);
  /* Check the end of ADC1 reset calibration register */
  while(ADC_GetResetCalibrationStatus(ADC1));

  /* Start ADC1 calibration */
  ADC_StartCalibration(ADC1);
  /* Check the end of ADC1 calibration */
  while(ADC_GetCalibrationStatus(ADC1));

  /* Start ADC1 Software Conversion so it's ready when the interrupt happens*/
  ADC_SoftwareStartConvCmd(ADC1, ENABLE);
}

void USART_Configuration(void)
{
  USART_InitTypeDef USART_InitStructure;
/* USARTx configured as follow:
      - BaudRate = 115200 baud
      - Word Length = 8 Bits
      - One Stop Bit
      - No parity
      - Hardware flow control disabled (RTS and CTS signals)
      - Receive and transmit enabled
*/
USART_InitStructure.USART_BaudRate = 115200;
USART_InitStructure.USART_WordLength = USART_WordLength_8b;
USART_InitStructure.USART_StopBits = USART_StopBits_1;
USART_InitStructure.USART_Parity = USART_Parity_No;
USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

STM_EVAL_COMInit(COM1, &USART_InitStructure);

}

/* ----------------------------------------------------------
    TIM3 configuration: Pulse measure input mode
    The external signal is connected to TIM3 CH2 pin (PA.07)
    CC1 is used: Reset on leading (neg) edge, capture and interrupt on trailing edge.
    Timer prescale is 24M / 24 = 1uS
 ------------------------------------------------------------ */
void TIM_Configuration(void)
{
	TIM_ICInitTypeDef  TIM_ICInitStructure;

	TIM_ICInitStructure.TIM_Channel = TIM_Channel_2;
//	TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Rising;
	TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Falling;
	TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;
	TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV1;
	TIM_ICInitStructure.TIM_ICFilter = 0x0;
	TIM_PWMIConfig(TIM3, &TIM_ICInitStructure);

	TIM3->PSC = 24 - 1;			/* Set timer clock to 24MHz / 24 = 1MHz = 1uS */

	/* Select the TIM3 Input Trigger: TI2FP2 */
	TIM_SelectInputTrigger(TIM3, TIM_TS_TI2FP2);
	/* Select the slave Mode: Reset Mode */
	TIM_SelectSlaveMode(TIM3, TIM_SlaveMode_Reset);
	/* Enable the Master/Slave Mode */
	TIM_SelectMasterSlaveMode(TIM3, TIM_MasterSlaveMode_Disable);
	/* Enable counter */
	TIM_Cmd(TIM3, ENABLE);
	/* Enable the CC1 Interrupt Request */
//  TIM_ITConfig(TIM3, TIM_IT_CC2, ENABLE);		/* Causes weird command delay */
	TIM_ITConfig(TIM3, TIM_IT_CC1, ENABLE);
}

/**
  * @brief  Inserts a coarse delay time.
  * @param  nCount: specifies the delay time length.
  * @retval None
  * 100 counts is about 45uS, so about .45us per count
  */
void Delay(__IO uint32_t nCount)
{
  for(; nCount != 0; nCount--);
}

/* Simple delay using the ms time tick interrupt */
void Delay_ms(__IO uint32_t nTime)
{
  TimingDelay = nTime;

  while(TimingDelay != 0);
}

/**
  * @brief  Decrements the TimingDelay variable.
  * @param  None
  * @retval None
  * Called by the time tick interrupt
  * Used by delay_ms()
  */

void TimingDelay_Decrement(void)
{
  if (TimingDelay != 0x00)
  {
    TimingDelay--;
  }
}

/**
  * @brief  Retargets the C library printf function to the USART.
  * @param  None
  * @retval None
  * I never got it working though. Someday...
  */
PUTCHAR_PROTOTYPE
{
  /* Place your implementation of fputc here */
  /* e.g. write a character to the USART */
  USART_SendData(EVAL_COM1, (uint8_t) ch);
  /* Loop until the end of transmission */
  while (USART_GetFlagStatus(EVAL_COM1, USART_FLAG_TC) == RESET)
  {}
  return ch;
}

#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {

  }
}
#endif


/* Send a string to the UART */
void putString(char *string)
{
	while (*string != 0)
	{
		putChar(*string++);
	}
}

/* Send a character to the UART */
void putChar(int32_t ch)
{
	  USART_SendData(EVAL_COM1, (uint8_t) ch);
	  /* Loop until the end of transmission */
	  while (USART_GetFlagStatus(EVAL_COM1, USART_FLAG_TC) == RESET)
	  {}
}

/**
  * @brief  returns a char from UARTx
  * @param  USARTx: Select the USART or the UART peripheral.
  *   This parameter can be one of the following values:
  *   USART1, USART2, USART3, UART4 or UART5.
  * @retval The received data. 0 if no character was returned
  */
uint16_t getChar(USART_TypeDef* USARTx)
{
	  /* Check the parameters */
	  assert_param(IS_USART_ALL_PERIPH(USARTx));

	  if (USART_GetFlagStatus(EVAL_COM1, USART_FLAG_RXNE) == RESET)
	  {
		  return 0x0000;
	  }
	  else
		  return USART_RecData(USARTx);
}


/**
  * @brief  Returns the most recent received data by the USARTx peripheral.
  * @param  USARTx: Select the USART or the UART peripheral.
  *   This parameter can be one of the following values:
  *   USART1, USART2, USART3, UART4 or UART5.
  * @retval The received data.
  */
uint16_t USART_RecData(USART_TypeDef* USARTx)
{
  /* Check the parameters */
  assert_param(IS_USART_ALL_PERIPH(USARTx));

  /* Receive Data */
  return (uint16_t)(USARTx->DR & (uint16_t)0x01FF);
}


